searchNodes=[{"doc":"Generates a Filter schema around an Ecto schema","ref":"Mix.Tasks.Phx.Bricks.Gen.Filter.html","title":"Mix.Tasks.Phx.Bricks.Gen.Filter","type":"task"},{"doc":"Generates a Query module that improves an Ecto.Query of provided schema","ref":"Mix.Tasks.Phx.Bricks.Gen.Query.html","title":"Mix.Tasks.Phx.Bricks.Gen.Query","type":"task"},{"doc":"An opinable set of proposed patters that helps to write reusable and no repetitive code for Contexts.MotivationAfter several years in Ruby on Rails developing I've got used to structure code folllowing the Single Responsibility Principle.Phoenix comes with the Context concept, a module that cares about expose an API of an application section to other sections.In a Context we usually have at least 6 actions for each defined schema (list_records/0, get_record!/1, create_record/1, update_record/2, delete_record/1, change_record/2).If we consider that all Business Logic could go inside the Context, it's possibile to have a module with hundreds of lines of code, making code mainteinance very hard to be guardanteed.The idea is to highlight common portion of code that can be extacted and moved into a separated module with only one responsibility and that could be reused in different contexts.List recordsThe method list_* has a default implementation that returns the list of associated record:def list_records do MyApp.Context.RecordSchema |&gt; MyApp.Repo.all() endLet's add now to the context the capability of filtering the collection according to an arbitrary set of scopes, calling the function in this way:iex&gt; Context.list_records(title_matches: &quot;value&quot;)A possible solution could be to delegate the query building into a separated RecordQuery moduledefmodule RecordQuery do def scope(list_of_filters) do RecordSchema |&gt; improve_query_with_filters(list_of_filters) end defp improve_query_with_filters(start_query, list_of_filters) do list_of_filters |&gt; Enum.reduce(start_query, fn scope, query -&gt; apply_scope(query, scope) end) end defp apply_scope(query, {:title_matches, &quot;value&quot;}) do query |&gt; where([q], ...) end defp apply_scope(query, {:price_lte, value}) do query |&gt; where([q], ...) end endand use it into the Contextdef list_records(scopes \\\\ []) do RecordQuery.scope(scopes) |&gt; Repo.all() end iex&gt; Context.list_records(title_matches: &quot;value&quot;, price_lte: 42)PhoenixBricks.ScopesUsing PhoenixBricks.Scopes it's possible to extend a module with all scope behaviours:defmodule RecordQuery do use PhoenixBricks.Scopes, schema: RecordSchema defp apply_scope(query, {:title_matches, &quot;value&quot;}) do query |&gt; where([q], ...) end endFilterAnother common feature is to filter records according to params provided through url params (for example after a submit in a search form).def index(conn, params) filters = Map.get(params, &quot;filters&quot;, %{}) colletion = Context.list_records_based_on_filters(filters) conn |&gt; assign(:collection, collection) ... endensuring to allow only specified filtersA possible implementation could be:defmodule RecordFilter do @search_filters [&quot;title_matches&quot;, &quot;price_lte&quot;] def convert_filters_to_scopes(filters) do filters |&gt; Enum.map(fn {name, value} -&gt; convert_filter_to_scope(name, value) end) end def convert_filter_to_scope(name, value) when name in @search_fields do {String.to_atom(name), value} end endThis way parameters are filtered and converted to a Keyword that is the common format for the RecordQuery described above.iex&gt; RecordFilter.convert_filters_to_scopes(%{&quot;title_matches&quot; =&gt; &quot;value&quot;, &quot;invalid_scope&quot; =&gt; &quot;value&quot;}) iex&gt; [title_matches: &quot;value&quot;]and we can rewrite the previous action emphasizing the params convertion and the collection filterdef index(conn, params) do filters = Map.get(params, &quot;filters&quot;, %{}) collection = filters |&gt; RecordFilter.convert_filters_to_scopes() |&gt; Context.list_records() conn |&gt; assign(:collection, collection) .... endThe last part is to build a search form. In order to achieve this, we can add schema functionality to RecordFilter module:defmodule RecordFilter do use Ecto.Schema embedded_schema do field :title_matches, :string end def changeset(filter, params) do filter |&gt; cast(params, [:title_matches]) end end def index(conn, params) do filters = Map.get(params, &quot;filters&quot;, %{}) filter_changeset = RecordFilter.changeset(%RecordFilter{}, filters) collection = filters |&gt; RecordFilter.convert_filters_to_scopes() |&gt; Context.list_records() conn |&gt; assign(:collection, collection) |&gt; assign(:filter_changeset, filter_changeset) end &lt;%= f = form_for @filter_changeset, .... %&gt; &lt;%= label f, :title_matches %&gt; &lt;%= text_input f, :title_matches %&gt; &lt;%= submit &quot;Filter results&quot; %&gt; &lt;% end %&gt;PhoenixBricks.FilterUsing PhoenixBricks.Filter module it's possible to extend a module with all filtering behaviours (define a changeset and the filter convertion)defmodule RecordFilter do use PhoenixBricks.Filter, filters: [ title_matches: :string ] endmaking available changeset/2 defined above and convert_filters_to_scopes/1","ref":"PhoenixBricks.html","title":"PhoenixBricks","type":"module"},{"doc":"Defines a Filter SchemaExamplesdefmodule RecordFilter do use PhoenixBricks.Filter, filters: [ field_matcher: :string ] endIt defines a schema which fields could be used in a search form.Search changesetdef index(conn, params) do filters = Map.get(params, &quot;filters&quot;, %{}) conn |&gt; assign(:changeset, RecordFilter.changeset(%RecordFilter{}, filters)) |&gt; render(&quot;index.html&quot;) end&lt;%= form_for @conn, Routes.session_path(@conn, :create), [method: :post, as: :user], fn f -&gt; %&gt; &lt;div class=&quot;form-group&quot;&gt; &lt;%= label f, :field_matcher %&gt; &lt;%= text_input f, :field_matcher %&gt; &lt;/div&gt; &lt;div class=&quot;form-group&quot;&gt;kab &lt;%= submit &quot;Search&quot; %&gt; &lt;/div&gt; &lt;% end %&gt;Convertion from map of params to a list of filtered scopesiex&gt; params = %{&quot;filters&quot; =&gt; %{&quot;field_matcher&quot; =&gt; &quot;value&quot;}} iex&gt; RecordFilter.convert_params_to_scopes(params) iex&gt; [field_matcher: &quot;value&quot;]","ref":"PhoenixBricks.Filter.html","title":"PhoenixBricks.Filter","type":"module"},{"doc":"Defines a common interface for adding scopes to a SchemaExamplesdefmodule RecordQuery do use ExBricks.Scopes, schema: Record endIt provides you a method scope that improve the empty scope of provided schema with additional scopes.scope/0 returns the empty scopeiex&gt; RecordQuery.scope() iex&gt; RecordWith scope/1 you can provide a list of scopes that improve the starting scopeiex&gt; scopes = [field1: {:eq, &quot;value&quot;}, field2: {:gte, 42}] iex&gt; RecordQuery.scope(scopes) iex&gt; #Ecto.Query&lt;from r0 in Record, where: r0.field1 == ^&quot;value&quot; and r0.field2 &gt;= 42&gt;If you need to improve an existing Ecto.Query you can use scope/2iex&gt; starting_scope = from(r in Record, where: r.field1 == &quot;value&quot;) iex&gt; scopes = [field2: {:gte, 42}] iex&gt; RecordQuery.scope(starting_scope, scopes) iex&gt; #Ecto.Query&lt;from r0 in Record, where: r0.field1 == ^&quot;value&quot; and r0.field2 &gt;= 42&gt;Built-in scopes:eqiex&gt; RecordQuery.scope(field: {:eq, &quot;value&quot;}) iex&gt; #Ecto.Query&lt;from r0 in Record, where: r0.fields == ^&quot;value&quot;&gt;:gtiex&gt; RecordQuery.scope(field: {:gt, &quot;value&quot;}) iex&gt; #Ecto.Query&lt;from r0 in Record, where: r0.fields &gt; ^&quot;value&quot;&gt;:ltiex&gt; RecordQuery.scope(field: {:lt, &quot;value&quot;}) iex&gt; #Ecto.Query&lt;from r0 in Record, where: r0.fields &lt; ^&quot;value&quot;&gt;:gteiex&gt; RecordQuery.scope(field: {:gte, &quot;value&quot;}) iex&gt; #Ecto.Query&lt;from r0 in Record, where: r0.fields &gt;= ^&quot;value&quot;&gt;:lteiex&gt; RecordQuery.scope(field: {:lte, &quot;value&quot;}) iex&gt; #Ecto.Query&lt;from r0 in Record, where: r0.fields &lt;= ^&quot;value&quot;&gt;:neqiex&gt; RecordQuery.scope(field: {:neq, &quot;value&quot;}) iex&gt; #Ecto.Query&lt;from r0 in Record, where: r0.fields != ^&quot;value&quot;&gt;:matchesiex&gt; RecordQuery.scope(field: {:matches, &quot;value&quot;}) iex&gt; #Ecto.Query&lt;from r0 in Record, where: ilike(r0.field, ^&quot;%value%&quot;)&gt;Customize scopesIf you need to define more comprehensive scopes you can improve the query adding new apply_scope/2 methodsdefmodule RecordQuery, do use ExBricks.Scopes, schema: Record defp apply_scope(query, {:name_matches, &quot;value&quot;}) do query |&gt; apply_scope(:name, {:matches, &quot;value&quot;}) end defp apply_scope(query, {:published}) do query |&gt; where([p], p.status == &quot;published&quot;) end end iex&gt; RecordQuery.scope(:published, name_matches: &quot;value&quot;) iex&gt; #Ecto.Query&lt;from r0 in Record, where: r0.status == &quot;published&quot; and ilike(r0.name, ^&quot;%value%&quot;)&gt;","ref":"PhoenixBricks.Scopes.html","title":"PhoenixBricks.Scopes","type":"module"},{"doc":"PhoenixBricksA set of proposed patterns to improve code organization for Phoenix application","ref":"readme.html","title":"PhoenixBricks","type":"extras"},{"doc":"To install PhoenixBricks, add it to your list of dependencies in mix.exs.def deps do [ {:phoenix_bricks, &quot;~&gt; 0.3.0&quot;} ] endOnce you've added PhoenixBricks to your list, update your dependencies by running:$ mix deps.get","ref":"readme.html#installation","title":"PhoenixBricks - Installation","type":"extras"},{"doc":"PhoenixBricks it is intented as a set of proposed pattern to organize code into a Phoenix application in a modular way.QueriesA query is a module that receives a list of atom/keywords and returns an Ecto.Query improved with provided scopesProductQuery.scope([title_matches: &quot;a value&quot;, price_is_greater_than: 1000]) =&gt; #Ecto.Query&lt;from p0 in PhoenixBricks.Catalogue.Product, where: ilike(p0.title, ^&quot;%a value%&quot;), where: p0.price &gt;= ^1000The idea is to have a query composer that could be easy to use like ActiveRecord.To generate a query for the Catalogue.Product schema with some additional custom scopes, simply run$ mix phx.bricks.gen.query Catalogue.Product name:matches:string price:lte:integerThe generated module will contain a set of default scopes for simple column matchers and the additional scopes provided in the command.defmodule PhoenixBricks.Catalogue.ProductQuery do @moduledoc false import Ecto.Query, warn: false def starting_scope do PhoenixBricks.Catalogue.Product end def scope(scopes, starting_scope) do scopes |&gt; Enum.reduce(starting_scope, fn scope, query -&gt; apply_scope(query, scope) end) end def scope(scopes) do scope(scopes, starting_scope()) end def scope do starting_scope() end defp apply_scope(query, {column, {:eq, value}}) do where(query, [q], field(q, ^column) == ^value) end defp apply_scope(query, {column, {:neq, value}}) do where(query, [q], field(q, ^column) != ^value) end defp apply_scope(query, {column, {:lte, value}}) do where(query, [q], field(q, ^column) &lt;= ^value) end defp apply_scope(query, {column, {:lt, value}}) do where(query, [q], field(q, ^column) &lt; ^value) end defp apply_scope(query, {column, {:gte, value}}) do where(query, [q], field(q, ^column) &gt;= ^value) end defp apply_scope(query, {column, {:gt, value}}) do where(query, [q], field(q, ^column) &gt; ^value) end defp apply_scope(query, {column, {:matches, value}}) do value = &quot;%\#{value}%&quot; where(query, [q], ilike(field(q, ^column), ^value)) end defp apply_scope(query, {:title_matches, value}) do apply_scope(query, {:title, {:matches, value}}) end defp apply_scope(query, {:price_gte, value}) do apply_scope(query, {:price, {:gte, value}}) end endUsing the ProductQuery module it's possible to rewrite the list_products/0 method of the context this way:defmodule PhoenixBricks.Catalogue do ... def list_products(scopes \\\\ []) do ProductQuery.scope(scopes) |&gt; Repo.all() end ... end [title_matches: &quot;a value&quot;] |&gt; Catalogue.list_products()If you wanto to add custom scopes you simply have to add new definitions of the apply_scope/2 method:defp apply_scope(query, :active) do from(q in query, where: q.active == true) end defp apply_scope(query, {:some_scope_name, value}) do from(query in q, ...) end ... Catalogue.list_products([ :active, title_matches: &quot;a value&quot;, some_scope_name: &quot;some value&quot; ])FiltersA filter is a module that receives a map of filters (for instance coming from a search form) and returns a list of allowed filters.%{&quot;filters&quot; =&gt; %{&quot;title_matches&quot; =&gt; &quot;a value&quot;, &quot;not_allowed&quot; =&gt; &quot;xxx&quot;}} |&gt; ProductFilter.params_to_scopes() =&gt; [title_matches: &quot;a value&quot;]To generate a module filter for the Catalogue.Product schema you simply have to run$ mix phx.bricks.gen.filters Catalogue.Product title:matches:string price:gte:integerthat will create a schema that we could use if a form for records filtering.defmodule CrudExample.Catalogue.ProductFilter do @moduledoc false use Ecto.Schema import Ecto.Changeset alias CrudExample.Catalogue.ProductFilter embedded_schema do field :title_matches, :string field :price_gte, :integer end def changeset(filter, attrs) do filter |&gt; cast(attrs, [:title_matches, :price_gte]) end def params_to_scopes(params) do filters = Map.get(params, &quot;filters&quot;, %{}) filter_changeset = changeset(%ProductFilter{}, filters) filter_changeset.changes |&gt; Enum.map(fn {name, value} -&gt; {name, value} end) end end# lib/phoenix_bricks/catalogue.ex def filter_products(params) do ProductFilter.changeset(%ProductFilter{}, params) end # lib/phoenix_bricks_web/controllers/products_controller.ex def index(conn, params) do filter_changeset = ProductFilter.changeset(%ProductFilter{}, params) products = params |&gt; ProductFilter.params_to_scopes() |&gt; ProductsQuery.scope() |&gt; Repo.all() # or # params # |&gt; ProductsFilter.params_to_scopes() # |&gt; Products.list_products() render( conn, &quot;index.html&quot;, products: products, filter_changeset: filter_changeset ) end # lib/phoenix_bricks_web/templates/products/index.html.eex &lt;%= form_for @filter_changeset, Routes.product_index_path(@conn, :index) method: :get do %&gt; &lt;%= label :title_matches %&gt; &lt;%= text_input :title_matches %&gt; &lt;% end %&gt; &lt;%= for product &lt;- @products do %&gt; ... &lt;% end %&gt;Query e FiltersThe 2 modules can be used in pipeline since the output format for filters are the input format of queries.%{&quot;filters&quot; =&gt; %{&quot;title_matches&quot; =&gt; &quot;title&quot;, &quot;price_gte&quot; =&gt; &quot;32&quot;}} |&gt; ProductFilter.params_to_scopes() |&gt; ProductQuery.scope() |&gt; Repo.all()","ref":"readme.html#getting-started","title":"PhoenixBricks - Getting started","type":"extras"},{"doc":"MIT LicenseCopyright (c) 2021 David LibreraPermission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the &quot;Software&quot;), to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.THE SOFTWARE IS PROVIDED &quot;AS IS&quot;, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.","ref":"license.html","title":"MIT License","type":"extras"}]